﻿////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Autodesk, Inc. All rights reserved 
// Written by Philippe Leefsma 2011 - ADN/Developer Technical Services
//
// This software is provided as is, without any warranty that it will work. You choose to use this tool at your own risk.
// Neither Autodesk nor the author Philippe Leefsma can be taken as responsible for any damage this tool can cause to 
// your data. Please always make a back up of your data prior to use this tool, as it will modify the documents involved 
// in the feature transformation.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
using System;
using System.IO;
using System.Reflection;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Configuration.Install;
using System.Runtime.InteropServices;

namespace ThreadModeler
{
    [RunInstaller(true)]
    public partial class AddInInstaller : Installer
    {
        [DllImport("kernel32.dll", SetLastError = true, 
            CallingConvention = CallingConvention.Winapi)]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool IsWow64Process([In] IntPtr hProcess, 
            [Out] out bool wow64Process);

        public AddInInstaller()
        {
            InitializeComponent();
        }

        public override void Install(IDictionary stateSaver)
        {
            base.Install(stateSaver);

            bool is64 = false;
            IsWow64Process(Process.GetCurrentProcess().Handle, 
                out is64);

            if (is64)
            {
                RegAsm64("/codebase");
            }
            else
            {
                RegistrationServices regsrv = 
                    new RegistrationServices();

                regsrv.RegisterAssembly(GetType().Assembly, 
                    AssemblyRegistrationFlags.SetCodeBase);
            }
        }

        public override void Uninstall(IDictionary savedState)
        {
            base.Uninstall(savedState);

            bool is64 = false;

            IsWow64Process(Process.GetCurrentProcess().Handle, 
                out is64);

            if (is64)
            {
                RegAsm64("/u");
            }
            else
            {
                RegistrationServices regsrv = 
                    new RegistrationServices();

                regsrv.UnregisterAssembly(GetType().Assembly);
            }

            base.Uninstall(savedState);
        }

        private static void RegAsm64(string parameters)
        {
            //.Net Framework Path
            string fmwk_path = Path.GetFullPath(
                Path.Combine(RuntimeEnvironment.GetRuntimeDirectory(), 
                "..\\.."));

            //RegAsm Path
            string regasm_path = string.Concat(fmwk_path,
                "\\Framework64\\",
                RuntimeEnvironment.GetSystemVersion(),
                "\\regasm.exe");

            if (!File.Exists(regasm_path))
            {
                MessageBox.Show("Failed to find RegAsm",
                    "Installer Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                return;
            }

            string addin_path = 
                Assembly.GetExecutingAssembly().Location;

            System.Diagnostics.Process process = 
                new System.Diagnostics.Process
            {
                StartInfo =
                {
                    CreateNoWindow = true,
                    ErrorDialog = false,
                    UseShellExecute = false,
                    FileName = regasm_path,
                    Arguments = string.Format("\"{0}\" {1}", 
                        addin_path, 
                        parameters)
                }
            };

            using (process)
            {
                process.Start();
                process.WaitForExit();
            }
        }
    }
}

